@@ -2,6 +2,7 @@
 pub enum PrefixUnaryOperator {
     Minus,
     Bang,
+    Not,
 }
 
 #[derive(PartialEq)]
@@ -11,6 +12,7 @@ pub enum ArithmeticOperator {
     Star,
     Slash,
     Modulus,
+    Exponentiation,
 }
 
 #[derive(PartialEq)]
@@ -25,14 +27,14 @@ pub enum ComparisonOperator {
 }
 
 #[derive(PartialEq)]
-pub enum LogicalOperator {
+pub enum BinaryLogicalOperator {
     Or,
     And,
     Xor,
 }
 
 #[derive(PartialEq)]
-pub enum BitwiseOperator {
+pub enum BinaryBitwiseOperator {
     Or,
     And,
     RightShift,
