@@ -14,6 +14,7 @@ pub enum StatementKind {
     GroupBy,
     AggregateFunction,
     GlobalVariable,
+    Into,
 }
 
 pub trait Statement {
@@ -34,7 +35,7 @@ pub struct GQLQuery {
     pub alias_table: HashMap<String, String>,
     pub has_aggregation_function: bool,
     pub has_group_by_statement: bool,
-    pub hidden_selections: Vec<String>,
+    pub hidden_selections: HashMap<String, Vec<String>>,
 }
 
 pub struct DoStatement {
@@ -57,10 +58,38 @@ pub enum Distinct {
     DistinctOn(Vec<String>),
 }
 
-pub struct SelectStatement {
+pub struct TableSelection {
     pub table_name: String,
-    pub fields_names: Vec<String>,
-    pub fields_values: Vec<Box<dyn Expression>>,
+    pub columns_names: Vec<String>,
+}
+
+#[derive(PartialEq)]
+pub enum JoinKind {
+    Cross,
+    Inner,
+    Left,
+    Right,
+    Default,
+}
+
+pub enum JoinOperand {
+    /// Used when JOIN is used first time on query, X JOIN Y,
+    OuterAndInner(String, String),
+    /// Used for JOIN that used afrer first time, JOIN Z
+    Inner(String),
+}
+
+pub struct Join {
+    pub operand: JoinOperand,
+    pub kind: JoinKind,
+    pub predicate: Option<Box<dyn Expression>>,
+}
+
+pub struct SelectStatement {
+    pub table_selections: Vec<TableSelection>,
+    pub joins: Vec<Join>,
+    pub selected_expr_titles: Vec<String>,
+    pub selected_expr: Vec<Box<dyn Expression>>,
     pub distinct: Distinct,
 }
 
@@ -199,6 +228,23 @@ impl Statement for GlobalVariableStatement {
     }
 }
 
+pub struct IntoStatement {
+    pub file_path: String,
+    pub lines_terminated: String,
+    pub fields_terminated: String,
+    pub enclosed: String,
+}
+
+impl Statement for IntoStatement {
+    fn as_any(&self) -> &dyn Any {
+        self
+    }
+
+    fn kind(&self) -> StatementKind {
+        StatementKind::Into
+    }
+}
+
 #[derive(Debug)]
 pub struct DescribeStatement {
     pub table_name: String,
