@@ -5,13 +5,17 @@ use std::collections::HashMap;
 use std::hash::DefaultHasher;
 use std::hash::Hash;
 use std::hash::Hasher;
+use std::io::Write;
 
+use gitql_ast::expression::Expression;
+use gitql_ast::expression::ExpressionKind;
 use gitql_ast::statement::AggregateValue;
 use gitql_ast::statement::AggregationsStatement;
 use gitql_ast::statement::DoStatement;
 use gitql_ast::statement::GlobalVariableStatement;
 use gitql_ast::statement::GroupByStatement;
 use gitql_ast::statement::HavingStatement;
+use gitql_ast::statement::IntoStatement;
 use gitql_ast::statement::LimitStatement;
 use gitql_ast::statement::OffsetStatement;
 use gitql_ast::statement::OrderByStatement;
@@ -24,10 +28,13 @@ use gitql_core::environment::Environment;
 use gitql_core::object::GitQLObject;
 use gitql_core::object::Group;
 use gitql_core::object::Row;
+use gitql_core::types::DataType;
 use gitql_core::value::Value;
 
 use crate::data_provider::DataProvider;
 use crate::engine_evaluator::evaluate_expression;
+use crate::engine_filter::apply_filter_operation;
+use crate::engine_join::apply_join_operation;
 
 #[allow(clippy::borrowed_box)]
 pub fn execute_statement(
@@ -36,7 +43,7 @@ pub fn execute_statement(
     data_provider: &Box<dyn DataProvider>,
     gitql_object: &mut GitQLObject,
     alias_table: &mut HashMap<String, String>,
-    hidden_selection: &Vec<String>,
+    hidden_selection: &HashMap<String, Vec<String>>,
 ) -> Result<(), String> {
     match statement.kind() {
         Do => {
@@ -101,6 +108,10 @@ pub fn execute_statement(
                 .unwrap();
             execute_aggregation_function_statement(env, statement, gitql_object, alias_table)
         }
+        Into => {
+            let statement = statement.as_any().downcast_ref::<IntoStatement>().unwrap();
+            execute_into_statement(statement, gitql_object)
+        }
         GlobalVariable => {
             let statement = statement
                 .as_any()
@@ -128,39 +139,132 @@ fn execute_select_statement(
     alias_table: &HashMap<String, String>,
     data_provider: &Box<dyn DataProvider>,
     gitql_object: &mut GitQLObject,
-    hidden_selections: &Vec<String>,
+    hidden_selections: &HashMap<String, Vec<String>>,
 ) -> Result<(), String> {
-    // Append hidden selection to the selected fields names
-    let mut fields_names = statement.fields_names.to_owned();
-    let mut hidden_selection_count = 0;
+    let mut selected_rows_per_table: HashMap<String, Vec<Row>> = HashMap::new();
+    let mut hidden_selection_count_per_table: HashMap<String, usize> = HashMap::new();
 
-    if !statement.table_name.is_empty() {
-        for hidden in hidden_selections {
-            if !fields_names.contains(hidden) {
-                fields_names.insert(0, hidden.to_string());
-                hidden_selection_count += 1;
+    let mut titles: Vec<String> = vec![];
+    let mut hidden_sum = 0;
+
+    for table_selection in &statement.table_selections {
+        // Select objects from the target table
+        let table_name = &table_selection.table_name;
+        let selected_columns = &mut table_selection.columns_names.to_owned();
+
+        // Insert Hidden selection items for this table first
+        let mut hidden_selection_count = 0;
+        if let Some(table_hidden_selection) = hidden_selections.get(table_name) {
+            for hidden_selection in table_hidden_selection {
+                if !selected_columns.contains(hidden_selection) {
+                    selected_columns.insert(0, hidden_selection.to_string());
+                    hidden_selection_count += 1;
+                }
+            }
+        }
+
+        hidden_selection_count_per_table.insert(table_name.to_string(), hidden_selection_count);
+
+        // Calculate list of titles once per table
+        let mut table_titles = vec![];
+        for selected_column in selected_columns.iter_mut() {
+            table_titles.push(get_column_name(alias_table, selected_column));
+        }
+
+        // Call the provider only if table name is not empty
+        let selected_rows: Vec<Row> = if table_name.is_empty() {
+            vec![Row { values: vec![] }]
+        } else {
+            data_provider.provide(table_name, selected_columns)?
+        };
+
+        selected_rows_per_table.insert(table_name.to_string(), selected_rows);
+
+        // Append hidden selection in the right position
+        // at the end all hidden selections will be first
+        let hidden_selection_titles = &table_titles[..hidden_selection_count];
+        titles.splice(hidden_sum..hidden_sum, hidden_selection_titles.to_vec());
+
+        // Non hidden selection should be inserted at the end
+        let selection_titles = &table_titles[hidden_selection_count..];
+        titles.extend_from_slice(selection_titles);
+        hidden_sum += hidden_selection_count;
+    }
+
+    gitql_object.titles.append(&mut titles);
+
+    // Apply joins operations if exists
+    let mut selected_rows: Vec<Row> = vec![];
+    apply_join_operation(
+        env,
+        &mut selected_rows,
+        &statement.joins,
+        &statement.table_selections,
+        &mut selected_rows_per_table,
+        &hidden_selection_count_per_table,
+        &gitql_object.titles,
+    )?;
+
+    // Execute Selected expressions if exists
+    if !statement.selected_expr.is_empty() {
+        execute_expression_selection(
+            env,
+            &mut selected_rows,
+            &gitql_object.titles,
+            &statement.selected_expr_titles,
+            &statement.selected_expr,
+        )?;
+    }
+
+    let main_group = Group {
+        rows: selected_rows,
+    };
+
+    gitql_object.groups.push(main_group);
+    Ok(())
+}
+
+#[inline(always)]
+fn execute_expression_selection(
+    env: &mut Environment,
+    selected_rows: &mut [Row],
+    object_titles: &[String],
+    selected_expr_titles: &[String],
+    selected_expr: &[Box<dyn Expression>],
+) -> Result<(), String> {
+    // Cache the index of each expression position to provide fast insertion
+    let mut titles_index_map: HashMap<String, usize> = HashMap::new();
+    for expr_column_title in selected_expr_titles {
+        let expr_title_index = object_titles
+            .iter()
+            .position(|r| r.eq(expr_column_title))
+            .unwrap();
+        titles_index_map.insert(expr_column_title.to_string(), expr_title_index);
+    }
+
+    for row in selected_rows.iter_mut() {
+        for (index, expr) in selected_expr.iter().enumerate() {
+            let expr_title = &selected_expr_titles[index];
+            let value_index = *titles_index_map.get(expr_title).unwrap();
+
+            if index < row.values.len() && row.values[value_index].data_type() != DataType::Null {
+                continue;
+            }
+
+            // Ignore evaluating expression if it symbol, that mean it a reference to aggregated value or function
+            let value = if expr.kind() == ExpressionKind::Symbol {
+                Value::Null
+            } else {
+                evaluate_expression(env, expr, object_titles, &row.values)?
+            };
+
+            if index >= row.values.len() {
+                row.values.push(value);
+            } else {
+                row.values[value_index] = value;
             }
         }
     }
-
-    // Calculate list of titles once
-    for field_name in &fields_names {
-        gitql_object
-            .titles
-            .push(get_column_name(alias_table, field_name));
-    }
-
-    // Select objects from the target table
-    let mut provided_object = data_provider.provide(
-        env,
-        &statement.table_name,
-        &fields_names,
-        &gitql_object.titles,
-        &statement.fields_values,
-        hidden_selection_count,
-    )?;
-
-    gitql_object.groups.append(&mut provided_object.groups);
     Ok(())
 }
 
@@ -169,31 +273,12 @@ fn execute_where_statement(
     statement: &WhereStatement,
     gitql_object: &mut GitQLObject,
 ) -> Result<(), String> {
-    if gitql_object.is_empty() {
-        return Ok(());
-    }
-
     // Perform where command only on the first group
     // because group by command not executed yet
-    let mut filtered_group: Group = Group { rows: vec![] };
-    let first_group = gitql_object.groups.first().unwrap().rows.iter();
-    for object in first_group {
-        let eval_result = evaluate_expression(
-            env,
-            &statement.condition,
-            &gitql_object.titles,
-            &object.values,
-        );
-        if eval_result.is_err() {
-            return Err(eval_result.err().unwrap());
-        }
-
-        if eval_result.ok().unwrap().as_bool() {
-            filtered_group.rows.push(Row {
-                values: object.values.clone(),
-            });
-        }
-    }
+    let condition = &statement.condition;
+    let main_group = &gitql_object.groups.first().unwrap().rows;
+    let rows = apply_filter_operation(env, condition, &gitql_object.titles, main_group)?;
+    let filtered_group: Group = Group { rows };
 
     // Update the main group with the filtered data
     gitql_object.groups.remove(0);
@@ -216,26 +301,11 @@ fn execute_having_statement(
     }
 
     // Perform where command only on the first group
-    // because groups are already merged
-    let mut filtered_group: Group = Group { rows: vec![] };
-    let first_group = gitql_object.groups.first().unwrap().rows.iter();
-    for object in first_group {
-        let eval_result = evaluate_expression(
-            env,
-            &statement.condition,
-            &gitql_object.titles,
-            &object.values,
-        );
-        if eval_result.is_err() {
-            return Err(eval_result.err().unwrap());
-        }
-
-        if eval_result.ok().unwrap().as_bool() {
-            filtered_group.rows.push(Row {
-                values: object.values.clone(),
-            });
-        }
-    }
+    // because group by command not executed yet
+    let condition = &statement.condition;
+    let main_group = &gitql_object.groups.first().unwrap().rows;
+    let rows = apply_filter_operation(env, condition, &gitql_object.titles, main_group)?;
+    let filtered_group: Group = Group { rows };
 
     // Update the main group with the filtered data
     gitql_object.groups.remove(0);
@@ -497,6 +567,56 @@ fn execute_aggregation_function_statement(
     Ok(())
 }
 
+fn execute_into_statement(
+    statement: &IntoStatement,
+    gitql_object: &mut GitQLObject,
+) -> Result<(), String> {
+    let mut buffer = String::new();
+
+    let line_terminated_by = &statement.lines_terminated;
+    let field_termianted_by = &statement.fields_terminated;
+    let enclosing = &statement.enclosed;
+
+    // Headers
+    let header = gitql_object.titles.join(field_termianted_by);
+    buffer.push_str(&header);
+    buffer.push_str(line_terminated_by);
+
+    // Rows of the main group
+    if let Some(main_group) = gitql_object.groups.first() {
+        for row in &main_group.rows {
+            let row_values: Vec<String> = row
+                .values
+                .iter()
+                .map(|r| value_to_string_with_optional_enclosing(r, enclosing))
+                .collect();
+            buffer.push_str(&row_values.join(field_termianted_by));
+            buffer.push_str(line_terminated_by);
+        }
+    }
+
+    let file_result = std::fs::File::create(statement.file_path.clone());
+    if let Err(error) = file_result {
+        return Err(error.to_string());
+    }
+
+    let mut file = file_result.ok().unwrap();
+    let write_result = file.write_all(buffer.as_bytes());
+    if let Err(error) = write_result {
+        return Err(error.to_string());
+    }
+
+    Ok(())
+}
+
+#[inline(always)]
+fn value_to_string_with_optional_enclosing(value: &Value, enclosed: &String) -> String {
+    if enclosed.is_empty() {
+        return value.to_string();
+    }
+    format!("{}{}{}", enclosed, value, enclosed)
+}
+
 pub fn execute_global_variable_statement(
     env: &mut Environment,
     statement: &GlobalVariableStatement,
