@@ -3,3 +3,5 @@ pub mod engine;
 pub mod engine_distinct;
 pub mod engine_evaluator;
 pub mod engine_executor;
+pub mod engine_filter;
+pub mod engine_join;
