@@ -22,12 +22,27 @@ pub enum TokenKind {
     Show,
     RegExp,
 
+    Join,
+    Left,
+    Right,
+    Cross,
+    Inner,
+    Outer,
+
     Case,
     When,
     Then,
     Else,
     End,
 
+    Into,
+    Outfile,
+    Dumpfile,
+    Lines,
+    Fields,
+    Enclosed,
+    Terminated,
+
     Between,
     DotDot,
 
@@ -42,19 +57,17 @@ pub enum TokenKind {
 
     As,
 
-    /// Left parenthesis `(`
     LeftParen,
-    /// Right parenthesis `)`
     RightParen,
-    /// Left bracket `[`
     LeftBracket,
-    /// Right bracket `]`
     RightBracket,
 
     LogicalOr,
     LogicalAnd,
     LogicalXor,
 
+    BitwiseNot,
+    BitwiseXor,
     BitwiseOr,
     BitwiseAnd,
     BitwiseRightShift,
@@ -71,6 +84,9 @@ pub enum TokenKind {
     False,
     Null,
 
+    Infinity,
+    NaN,
+
     Colon,
     ColonEqual,
 
@@ -79,6 +95,7 @@ pub enum TokenKind {
     Star,
     Slash,
     Percentage,
+    Caret,
 
     Comma,
     Dot,
@@ -313,6 +330,42 @@ pub fn tokenize(script: String) -> Result<Vec<Token>, Box<Diagnostic>> {
             continue;
         }
 
+        // Caret
+        if char == '^' {
+            let location = Location {
+                start: column_start,
+                end: position,
+            };
+
+            let token = Token {
+                location,
+                kind: TokenKind::Caret,
+                literal: "^".to_owned(),
+            };
+
+            tokens.push(token);
+            position += 1;
+            continue;
+        }
+
+        // Bitwise NOT
+        if char == '~' {
+            let location = Location {
+                start: column_start,
+                end: position,
+            };
+
+            let token = Token {
+                location,
+                kind: TokenKind::BitwiseNot,
+                literal: "~".to_owned(),
+            };
+
+            tokens.push(token);
+            position += 1;
+            continue;
+        }
+
         // Or
         if char == '|' {
             let location = Location {
@@ -369,7 +422,7 @@ pub fn tokenize(script: String) -> Result<Vec<Token>, Box<Diagnostic>> {
         }
 
         // xor
-        if char == '^' {
+        if char == '#' {
             let location = Location {
                 start: column_start,
                 end: position,
@@ -377,8 +430,8 @@ pub fn tokenize(script: String) -> Result<Vec<Token>, Box<Diagnostic>> {
 
             let token = Token {
                 location,
-                kind: TokenKind::LogicalXor,
-                literal: "^".to_owned(),
+                kind: TokenKind::BitwiseXor,
+                literal: "#".to_owned(),
             };
 
             tokens.push(token);
@@ -1084,6 +1137,23 @@ fn resolve_symbol_kind(literal: String) -> TokenKind {
         "show" => TokenKind::Show,
         "regexp" => TokenKind::RegExp,
 
+        // Joins
+        "join" => TokenKind::Join,
+        "left" => TokenKind::Left,
+        "right" => TokenKind::Right,
+        "cross" => TokenKind::Cross,
+        "inner" => TokenKind::Inner,
+        "outer" => TokenKind::Outer,
+
+        // Select into
+        "into" => TokenKind::Into,
+        "outfile" => TokenKind::Outfile,
+        "dumpfile" => TokenKind::Dumpfile,
+        "lines" => TokenKind::Lines,
+        "fields" => TokenKind::Fields,
+        "enclosed" => TokenKind::Enclosed,
+        "terminated" => TokenKind::Terminated,
+
         // Integer division and Modulo operator
         "div" => TokenKind::Slash,
         "mod" => TokenKind::Percentage,
@@ -1098,6 +1168,9 @@ fn resolve_symbol_kind(literal: String) -> TokenKind {
         "false" => TokenKind::False,
         "null" => TokenKind::Null,
 
+        "infinity" => TokenKind::Infinity,
+        "nan" => TokenKind::NaN,
+
         // As for alias
         "as" => TokenKind::As,
 
