@@ -105,6 +105,9 @@ pub fn standard_functions() -> &'static HashMap<&'static str, Function> {
 
         // Array Functions
         map.insert("array_length", array_length);
+        map.insert("array_shuffle", array_shuffle);
+        map.insert("array_position", array_position);
+        map.insert("array_dims", array_dims);
         map
     })
 }
@@ -697,7 +700,27 @@ pub fn standard_function_signatures() -> &'static HashMap<&'static str, Signatur
                 return_type: DataType::Integer,
             },
         );
-
+        map.insert(
+            "array_shuffle",
+            Signature {
+                parameters: vec![DataType::Array(Box::new(DataType::Any))],
+                return_type: DataType::Dynamic(same_type_as_first_parameter),
+            },
+        );
+        map.insert(
+            "array_position",
+            Signature {
+                parameters: vec![DataType::Array(Box::new(DataType::Any)), DataType::Any],
+                return_type: DataType::Integer,
+            },
+        );
+        map.insert(
+            "array_dims",
+            Signature {
+                parameters: vec![DataType::Array(Box::new(DataType::Any))],
+                return_type: DataType::Text,
+            },
+        );
         map
     })
 }
