@@ -1,13 +1,6 @@
-use gitql_ast::expression::Expression;
-use gitql_ast::expression::SymbolExpression;
-use gitql_core::environment::Environment;
-use gitql_core::object::GitQLObject;
-use gitql_core::object::Group;
 use gitql_core::object::Row;
 use gitql_core::value::Value;
-use gitql_engine::data_provider::select_values;
 use gitql_engine::data_provider::DataProvider;
-use gitql_engine::engine_evaluator::evaluate_expression;
 use gix::refs::Category;
 
 pub struct GitDataProvider {
@@ -21,130 +14,50 @@ impl GitDataProvider {
 }
 
 impl DataProvider for GitDataProvider {
-    fn provide(
-        &self,
-        env: &mut Environment,
-        table: &str,
-        fields_names: &[String],
-        titles: &[String],
-        fields_values: &[Box<dyn Expression>],
-        counter: i64,
-    ) -> Result<GitQLObject, String> {
-        let mut groups: Vec<Group> = vec![];
+    fn provide(&self, table: &str, selected_columns: &[String]) -> Result<Vec<Row>, String> {
+        let mut rows: Vec<Row> = vec![];
 
         for repository in &self.repos {
-            let mut repository_group = select_gql_objects(
-                env,
-                repository,
-                table.to_string(),
-                fields_names,
-                titles,
-                fields_values,
-                counter,
-            )?;
-
-            if groups.is_empty() {
-                groups.push(repository_group);
-            } else {
-                groups[0].rows.append(&mut repository_group.rows);
-            }
+            let mut repo_rows =
+                select_gql_objects(repository, table.to_string(), selected_columns)?;
+            rows.append(&mut repo_rows);
         }
 
-        Ok(GitQLObject {
-            titles: titles.to_vec(),
-            groups,
-        })
+        Ok(rows)
     }
 }
 
 fn select_gql_objects(
-    env: &mut Environment,
     repo: &gix::Repository,
     table: String,
-    fields_names: &[String],
-    titles: &[String],
-    fields_values: &[Box<dyn Expression>],
-    hidden_selection_count: i64,
-) -> Result<Group, String> {
+    selected_columns: &[String],
+) -> Result<Vec<Row>, String> {
     match table.as_str() {
-        "refs" => select_references(
-            env,
-            repo,
-            fields_names,
-            titles,
-            fields_values,
-            hidden_selection_count,
-        ),
-        "commits" => select_commits(
-            env,
-            repo,
-            fields_names,
-            titles,
-            fields_values,
-            hidden_selection_count,
-        ),
-        "branches" => select_branches(
-            env,
-            repo,
-            fields_names,
-            titles,
-            fields_values,
-            hidden_selection_count,
-        ),
-        "diffs" => select_diffs(
-            env,
-            repo,
-            fields_names,
-            titles,
-            fields_values,
-            hidden_selection_count,
-        ),
-        "tags" => select_tags(
-            env,
-            repo,
-            fields_names,
-            titles,
-            fields_values,
-            hidden_selection_count,
-        ),
-        _ => select_values(env, titles, fields_values),
+        "refs" => select_references(repo, selected_columns),
+        "commits" => select_commits(repo, selected_columns),
+        "branches" => select_branches(repo, selected_columns),
+        "diffs" => select_diffs(repo, selected_columns),
+        "tags" => select_tags(repo, selected_columns),
+        _ => Ok(vec![Row { values: vec![] }]),
     }
 }
 
 fn select_references(
-    env: &mut Environment,
     repo: &gix::Repository,
-    fields_names: &[String],
-    titles: &[String],
-    fields_values: &[Box<dyn Expression>],
-    hidden_selection_count: i64,
-) -> Result<Group, String> {
-    let repo_path = repo.path().to_str().unwrap().to_string();
-
-    let mut rows: Vec<Row> = vec![];
+    selected_columns: &[String],
+) -> Result<Vec<Row>, String> {
     let git_references = repo.references();
-    if git_references.is_err() {
-        return Ok(Group { rows });
+    if let Err(error) = git_references {
+        return Err(error.to_string());
     }
 
+    let repo_path = repo.path().to_str().unwrap().to_string();
     let references = git_references.ok().unwrap();
-    let names_len = fields_names.len() as i64;
-    let values_len = fields_values.len() as i64;
-    let padding = names_len - values_len;
+    let mut rows: Vec<Row> = vec![];
 
     for reference in references.all().unwrap().flatten() {
-        let mut values: Vec<Value> = Vec::with_capacity(fields_names.len());
-        for index in 0..names_len {
-            if index >= hidden_selection_count && (index - padding) >= 0 {
-                let value = &fields_values[(index - padding) as usize];
-                if value.as_any().downcast_ref::<SymbolExpression>().is_none() {
-                    let evaluated = evaluate_expression(env, value, titles, &values)?;
-                    values.push(evaluated);
-                    continue;
-                }
-            }
-
-            let field_name = &fields_names[index as usize];
+        let mut values: Vec<Value> = Vec::with_capacity(selected_columns.len());
+        for field_name in selected_columns {
             if field_name == "name" {
                 let name = reference
                     .name()
@@ -190,89 +103,65 @@ fn select_references(
         rows.push(row);
     }
 
-    Ok(Group { rows })
+    Ok(rows)
 }
 
-fn select_commits(
-    env: &mut Environment,
-    repo: &gix::Repository,
-    fields_names: &[String],
-    titles: &[String],
-    fields_values: &[Box<dyn Expression>],
-    hidden_selection_count: i64,
-) -> Result<Group, String> {
-    let repo_path = repo.path().to_str().unwrap().to_string();
-
-    let mut rows: Vec<Row> = vec![];
+fn select_commits(repo: &gix::Repository, selected_columns: &[String]) -> Result<Vec<Row>, String> {
     let head_id = repo.head_id();
-    if head_id.is_err() {
-        return Ok(Group { rows });
+    if let Err(error) = head_id {
+        return Err(error.to_string());
     }
 
+    let repo_path = repo.path().to_str().unwrap().to_string();
     let revwalk = head_id.unwrap().ancestors().all().unwrap();
-
-    let names_len = fields_names.len() as i64;
-    let values_len = fields_values.len() as i64;
-    let padding = names_len - values_len;
+    let mut rows: Vec<Row> = vec![];
 
     for commit_info in revwalk {
         let commit_info = commit_info.unwrap();
         let commit = repo.find_object(commit_info.id).unwrap().into_commit();
         let commit = commit.decode().unwrap();
 
-        let mut values: Vec<Value> = Vec::with_capacity(fields_names.len());
-
-        for index in 0..names_len {
-            if index >= hidden_selection_count && (index - padding) >= 0 {
-                let value = &fields_values[(index - padding) as usize];
-                if value.as_any().downcast_ref::<SymbolExpression>().is_none() {
-                    let evaluated = evaluate_expression(env, value, titles, &values)?;
-                    values.push(evaluated);
-                    continue;
-                }
-            }
-
-            let field_name = &fields_names[index as usize];
-
-            if field_name == "commit_id" {
+        let mut values: Vec<Value> = Vec::with_capacity(selected_columns.len());
+        for column_name in selected_columns {
+            if column_name == "commit_id" {
                 let commit_id = Value::Text(commit_info.id.to_string());
                 values.push(commit_id);
                 continue;
             }
 
-            if field_name == "author_name" {
+            if column_name == "author_name" {
                 values.push(Value::Text(commit.author().name.to_string()));
                 continue;
             }
 
-            if field_name == "author_email" {
+            if column_name == "author_email" {
                 values.push(Value::Text(commit.author().email.to_string()));
                 continue;
             }
 
-            if field_name == "committer_name" {
+            if column_name == "committer_name" {
                 values.push(Value::Text(commit.committer().name.to_string()));
                 continue;
             }
 
-            if field_name == "committer_email" {
+            if column_name == "committer_email" {
                 values.push(Value::Text(commit.committer().email.to_string()));
                 continue;
             }
 
-            if field_name == "title" {
+            if column_name == "title" {
                 let summary = Value::Text(commit.message().summary().to_string());
                 values.push(summary);
                 continue;
             }
 
-            if field_name == "message" {
+            if column_name == "message" {
                 let message = Value::Text(commit.message.to_string());
                 values.push(message);
                 continue;
             }
 
-            if field_name == "datetime" {
+            if column_name == "datetime" {
                 let time_stamp = commit_info
                     .commit_time
                     .unwrap_or_else(|| commit.time().seconds);
@@ -280,12 +169,12 @@ fn select_commits(
                 continue;
             }
 
-            if field_name == "parents_count" {
+            if column_name == "parents_count" {
                 values.push(Value::Integer(commit.parents.len() as i64));
                 continue;
             }
 
-            if field_name == "repo" {
+            if column_name == "repo" {
                 values.push(Value::Text(repo_path.to_string()));
                 continue;
             }
@@ -297,17 +186,13 @@ fn select_commits(
         rows.push(row);
     }
 
-    Ok(Group { rows })
+    Ok(rows)
 }
 
 fn select_branches(
-    env: &mut Environment,
     repo: &gix::Repository,
-    fields_names: &[String],
-    titles: &[String],
-    fields_values: &[Box<dyn Expression>],
-    hidden_selection_count: i64,
-) -> Result<Group, String> {
+    selected_columns: &[String],
+) -> Result<Vec<Row>, String> {
     let mut rows: Vec<Row> = vec![];
 
     let repo_path = repo.path().to_str().unwrap().to_string();
@@ -316,42 +201,28 @@ fn select_branches(
     let remote_branches = platform.remote_branches().unwrap();
     let local_and_remote_branches = local_branches.chain(remote_branches);
     let head_ref_result = repo.head_ref();
-    if head_ref_result.is_err() {
-        return Ok(Group { rows });
+    if let Err(error) = head_ref_result {
+        return Err(error.to_string());
     }
 
     let head_ref_option = head_ref_result.unwrap();
     if head_ref_option.is_none() {
-        return Ok(Group { rows });
+        return Ok(rows);
     }
 
     let head_ref = head_ref_option.unwrap();
 
-    let names_len = fields_names.len() as i64;
-    let values_len = fields_values.len() as i64;
-    let padding = names_len - values_len;
-
     for mut branch in local_and_remote_branches.flatten() {
-        let mut values: Vec<Value> = Vec::with_capacity(fields_names.len());
+        let mut values: Vec<Value> = Vec::with_capacity(selected_columns.len());
 
-        for index in 0..names_len {
-            if index >= hidden_selection_count && (index - padding) >= 0 {
-                let value = &fields_values[(index - padding) as usize];
-                if value.as_any().downcast_ref::<SymbolExpression>().is_none() {
-                    let evaluated = evaluate_expression(env, value, titles, &values)?;
-                    values.push(evaluated);
-                    continue;
-                }
-            }
-
-            let field_name = &fields_names[index as usize];
-            if field_name == "name" {
+        for column_name in selected_columns {
+            if column_name == "name" {
                 let branch_name = branch.name().as_bstr().to_string();
                 values.push(Value::Text(branch_name));
                 continue;
             }
 
-            if field_name == "commit_count" {
+            if column_name == "commit_count" {
                 let commit_count = if let Some(id) = branch.try_id() {
                     if let Ok(revwalk) = id.ancestors().all() {
                         revwalk.count() as i64
@@ -365,7 +236,7 @@ fn select_branches(
                 continue;
             }
 
-            if field_name == "updated" {
+            if column_name == "updated" {
                 if let Ok(top_commit_id) = branch.peel_to_id_in_place() {
                     let revwalk = top_commit_id.ancestors().all().unwrap();
                     if let Some(commit_info) = revwalk.into_iter().next() {
@@ -387,12 +258,12 @@ fn select_branches(
                 continue;
             }
 
-            if field_name == "is_head" {
+            if column_name == "is_head" {
                 values.push(Value::Boolean(branch.inner == head_ref.inner));
                 continue;
             }
 
-            if field_name == "is_remote" {
+            if column_name == "is_remote" {
                 let is_remote = branch
                     .name()
                     .category()
@@ -401,7 +272,7 @@ fn select_branches(
                 continue;
             }
 
-            if field_name == "repo" {
+            if column_name == "repo" {
                 values.push(Value::Text(repo_path.to_string()));
                 continue;
             }
@@ -413,78 +284,66 @@ fn select_branches(
         rows.push(row);
     }
 
-    Ok(Group { rows })
+    Ok(rows)
 }
 
-fn select_diffs(
-    env: &mut Environment,
-    repo: &gix::Repository,
-    fields_names: &[String],
-    titles: &[String],
-    fields_values: &[Box<dyn Expression>],
-    hidden_selection_count: i64,
-) -> Result<Group, String> {
+fn select_diffs(repo: &gix::Repository, selected_columns: &[String]) -> Result<Vec<Row>, String> {
     let repo = {
         let mut repo = repo.clone();
         repo.object_cache_size_if_unset(4 * 1024 * 1024);
         repo
     };
 
-    let mut rows: Vec<Row> = vec![];
     let revwalk = repo.head_id().unwrap().ancestors().all().unwrap();
     let repo_path = repo.path().to_str().unwrap().to_string();
 
     let mut rewrite_cache = repo
         .diff_resource_cache(gix::diff::blob::pipeline::Mode::ToGit, Default::default())
         .unwrap();
-    let mut diff_cache = rewrite_cache.clone();
 
-    let names_len = fields_names.len() as i64;
-    let values_len = fields_values.len() as i64;
-    let padding = names_len - values_len;
+    let mut diff_cache = rewrite_cache.clone();
+    let mut rows: Vec<Row> = vec![];
 
     for commit_info in revwalk {
         let commit_info = commit_info.unwrap();
         let commit = commit_info.id().object().unwrap().into_commit();
+        let commit_ref = commit.decode().unwrap();
+        let mut values: Vec<Value> = Vec::with_capacity(selected_columns.len());
 
-        let mut values: Vec<Value> = Vec::with_capacity(fields_names.len());
-
-        for index in 0..names_len {
-            if index >= hidden_selection_count && (index - padding) >= 0 {
-                let value = &fields_values[(index - padding) as usize];
-                if value.as_any().downcast_ref::<SymbolExpression>().is_none() {
-                    let evaluated = evaluate_expression(env, value, titles, &values)?;
-                    values.push(evaluated);
-                    continue;
-                }
-            }
-
-            let field_name = &fields_names[index as usize];
-            if field_name == "commit_id" {
+        for column_name in selected_columns {
+            if column_name == "commit_id" {
                 values.push(Value::Text(commit_info.id.to_string()));
                 continue;
             }
 
-            if field_name == "name" {
-                let name = commit.author().unwrap().name.to_string();
+            if column_name == "name" {
+                let name = commit_ref.author().name.to_string();
                 values.push(Value::Text(name));
                 continue;
             }
 
-            if field_name == "email" {
-                let email = commit.author().unwrap().email.to_string();
+            if column_name == "email" {
+                let email = commit_ref.author().email.to_string();
                 values.push(Value::Text(email));
                 continue;
             }
 
-            if field_name == "repo" {
+            if column_name == "datetime" {
+                let time_stamp = commit_info
+                    .commit_time
+                    .unwrap_or_else(|| commit_ref.time().seconds);
+                values.push(Value::DateTime(time_stamp));
+                continue;
+            }
+
+            if column_name == "repo" {
                 values.push(Value::Text(repo_path.to_string()));
                 continue;
             }
 
-            if field_name == "insertions"
-                || field_name == "deletions"
-                || field_name == "files_changed"
+            if column_name == "insertions"
+                || column_name == "deletions"
+                || column_name == "files_changed"
             {
                 let current = commit.tree().unwrap();
                 let previous = commit_info
@@ -494,7 +353,7 @@ fn select_diffs(
                     .unwrap_or_else(|| repo.empty_tree());
 
                 let select_insertions_or_deletions =
-                    field_name == "insertions" || field_name == "deletions";
+                    column_name == "insertions" || column_name == "deletions";
 
                 rewrite_cache.clear_resource_cache();
                 diff_cache.clear_resource_cache();
@@ -522,17 +381,17 @@ fn select_diffs(
                     )
                     .unwrap();
 
-                if field_name == "insertions" {
+                if column_name == "insertions" {
                     values.push(Value::Integer(insertions as i64));
                     continue;
                 }
 
-                if field_name == "deletions" {
+                if column_name == "deletions" {
                     values.push(Value::Integer(deletions as i64));
                     continue;
                 }
 
-                if field_name == "files_changed" {
+                if column_name == "files_changed" {
                     values.push(Value::Integer(files_changed as i64));
                     continue;
                 }
@@ -545,42 +404,20 @@ fn select_diffs(
         rows.push(row);
     }
 
-    Ok(Group { rows })
+    Ok(rows)
 }
 
-fn select_tags(
-    env: &mut Environment,
-    repo: &gix::Repository,
-    fields_names: &[String],
-    titles: &[String],
-    fields_values: &[Box<dyn Expression>],
-    hidden_selection_count: i64,
-) -> Result<Group, String> {
+fn select_tags(repo: &gix::Repository, selected_columns: &[String]) -> Result<Vec<Row>, String> {
     let platform = repo.references().unwrap();
     let tag_names = platform.tags().unwrap();
     let repo_path = repo.path().to_str().unwrap().to_string();
-
-    let names_len = fields_names.len() as i64;
-    let values_len = fields_values.len() as i64;
-    let padding = names_len - values_len;
-
     let mut rows: Vec<Row> = vec![];
 
     for tag_ref in tag_names.flatten() {
-        let mut values: Vec<Value> = Vec::with_capacity(fields_names.len());
+        let mut values: Vec<Value> = Vec::with_capacity(selected_columns.len());
 
-        for index in 0..names_len {
-            if index >= hidden_selection_count && (index - padding) >= 0 {
-                let value = &fields_values[(index - padding) as usize];
-                if value.as_any().downcast_ref::<SymbolExpression>().is_none() {
-                    let evaluated = evaluate_expression(env, value, titles, &values)?;
-                    values.push(evaluated);
-                    continue;
-                }
-            }
-
-            let field_name = &fields_names[index as usize];
-            if field_name == "name" {
+        for column_name in selected_columns {
+            if column_name == "name" {
                 let tag_name = tag_ref
                     .name()
                     .category_and_short_name()
@@ -589,7 +426,7 @@ fn select_tags(
                 continue;
             }
 
-            if field_name == "repo" {
+            if column_name == "repo" {
                 values.push(Value::Text(repo_path.to_string()));
                 continue;
             }
@@ -601,5 +438,5 @@ fn select_tags(
         rows.push(row);
     }
 
-    Ok(Group { rows })
+    Ok(rows)
 }
